@@ -1,20 +1,19 @@
 # SPDX-License-Identifier: Apache-2.0
 """New flow implementation for SDG Hub.
 
-This module provides a redesigned Flow class with metadata support,
-dual initialization modes, and runtime parameter overrides.
+This module provides a redesigned Flow class with metadata support
+and dual initialization modes.
 """
 
 # Local
 from .base import Flow
-from .metadata import FlowMetadata, FlowParameter
+from .metadata import FlowMetadata
 from .registry import FlowRegistry
 from .validation import FlowValidator
 
 __all__ = [
     "Flow",
     "FlowMetadata",
-    "FlowParameter",
     "FlowRegistry",
     "FlowValidator",
 ]
